// Sample code: Using the GPU/CUDA backend detect largest face in an image already loaded in the graphics card's memory.
// g++ ./face_detect_image_in_vram.cpp -I ../3rd_party_libs/opencv-4.1.1/build_amd64_cuda/packaged/usr/local/include/opencv4/ -L ./ -Wl,-rpath=/home/hesham/code/trueface/c-sdks/build/ libopencv_core.so.4.1 libopencv_imgproc.so.4.1 libopencv_imgcodecs.so libopencv_cudev.so.4.1 libopencv_cudaimgproc.so.4.1 libopencv_cudaarithm.so.4.1 libtf.so /usr/local/cuda/lib64/libcudart.so -fopenmp
// LD_LIBRARY_PATH=/home/hesham/code/trueface/c-sdks/build ./a.out

#include "../../tf_sdk.h"
#include <iostream>
#include <vector>
#include "opencv2/opencv.hpp"
#include "opencv2/core/core.hpp"
#include "opencv2/imgcodecs.hpp"
#include "opencv2/imgproc/imgproc.hpp"
#include "opencv2/core/cuda.hpp"

using namespace std;

typedef std::chrono::high_resolution_clock Clock;

int main() {

    Trueface::ConfigurationOptions options;
    options.frModel = Trueface::FacialRecognitionModel::FULL;
    options.enableGPU = true;
    options.deviceIndex = 0;

    Trueface::SDK tfSdk(options);

    bool valid = tfSdk.setLicense("${LICENSE_CODE}");

    if (valid == false) {
        std::cout << "Error: the provided license is invalid." << std::endl;
        return 1;
    }

    cv::cuda::printShortCudaDeviceInfo(cv::cuda::getDevice());

    vector<Trueface::Faceprint> faceprints1;
    vector<Trueface::Faceprint> faceprints2;

    {
        // using opencv load the image in vram
        cv::Mat img = cv::imread("../sample_code/images/brad_pitt_1.jpg");
        cv::cuda::GpuMat mat;
        mat.upload(img);
        uchar* ptr = mat.data;

        cout<<static_cast<void*>(ptr)<<endl;
        cout<<mat.rows<<endl;
        cout<<mat.cols<<endl;
        cout<<img.rows<<endl;
        cout<<img.cols<<endl;

        Trueface::ErrorCode errorCode = tfSdk.setImage(ptr, img.cols, img.rows, Trueface::ColorCode::bgr, mat.step);

        if (errorCode != Trueface::ErrorCode::NO_ERROR) {
            std::cout<<"Error: could not load the image"<<std::endl;
            return 1;
        }

        Trueface::FaceBoxAndLandmarks faceBoxAndLandmarks;
        bool found = false;
        tfSdk.detectLargestFace(faceBoxAndLandmarks, found);
        if (found == false) {
            std::cout<<"Error: could not detect a face"<<std::endl;
            return 1;
        } else {
            cout<<faceBoxAndLandmarks.topLeft.x<<endl;
            cout<<faceBoxAndLandmarks.topLeft.y<<endl;
            cout<<faceBoxAndLandmarks.bottomRight.x<<endl;
            cout<<faceBoxAndLandmarks.bottomRight.y<<endl;
        }

        cv::cuda::GpuMat chipGpu(1, 112*112, CV_8UC3);
        tfSdk.extractAlignedFace(faceBoxAndLandmarks, chipGpu.data);

        vector<uint8_t*> alignedFaceImages;
        alignedFaceImages.push_back(chipGpu.data);
        auto res = tfSdk.getFaceFeatureVectors(alignedFaceImages, faceprints1);
    }
    {
        // using opencv load the image in vram
        cv::Mat img = cv::imread("../sample_code/images/tom_cruise_1.jpg");
        cv::cuda::GpuMat mat;
        mat.upload(img);
        uchar* ptr = mat.data;

        cout<<static_cast<void*>(ptr)<<endl;
        cout<<mat.rows<<endl;
        cout<<mat.cols<<endl;
        cout<<img.rows<<endl;
        cout<<img.cols<<endl;

        Trueface::ErrorCode errorCode = tfSdk.setImage(ptr, img.cols, img.rows, Trueface::ColorCode::bgr, mat.step);

        if (errorCode != Trueface::ErrorCode::NO_ERROR) {
            std::cout<<"Error: could not load the image"<<std::endl;
            return 1;
        }

        Trueface::FaceBoxAndLandmarks faceBoxAndLandmarks;
        bool found = false;
        tfSdk.detectLargestFace(faceBoxAndLandmarks, found);
        if (found == false) {
            std::cout<<"Error: could not detect a face"<<std::endl;
            return 1;
        } else {
            cout<<faceBoxAndLandmarks.topLeft.x<<endl;
            cout<<faceBoxAndLandmarks.topLeft.y<<endl;
            cout<<faceBoxAndLandmarks.bottomRight.x<<endl;
            cout<<faceBoxAndLandmarks.bottomRight.y<<endl;
        }

        cv::cuda::GpuMat chipGpu(1, 112*112, CV_8UC3);
        tfSdk.extractAlignedFace(faceBoxAndLandmarks, chipGpu.data);

        vector<uint8_t*> alignedFaceImages;
        alignedFaceImages.push_back(chipGpu.data);

        auto res = tfSdk.getFaceFeatureVectors(alignedFaceImages, faceprints2);
    }

    float prob, cos;
    tfSdk.getSimilarity(faceprints1[0], faceprints2[0], prob, cos);
    cout<<"prob:"<<prob<<endl;
    cout<<"cos:"<<cos<<endl;

    return 0;
}